/*
	File: History.r

	Copyright (c) 1990-1, Thomas Knoll.
	Copyright (c) 1992-6, Adobe Systems Incorporated.
	All rights reserved.
	
	Rez source file for History export example.
*/

/********************************************************************************/
/* Defines required for include files */

#define plugInName			"History"
#define VersionString		"4.0"

/********************************************************************************/

#if Macintosh
#include "Types.r"
#include "SysTypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/********************************************************************************/

#define plugInComment 		"history example export plug-in"
#define vendorName			"AdobeSDK"
#define ourSuiteID			'sdKB'
#define ourClassID			'hisT'
#define ourEventID			typeNull // must be this

#define kNoHistories		ResourceID+100
#define kHistStatus			kNoHistories+1

/********************************************************************************/

resource 'PiPL' (ResourceID, purgeable)
{
	{
		Kind { Export },
		Name { plugInName "..." },
		Version { (latestExportVersion << 16) | latestExportSubVersion },

		#if Macintosh
		Code68K { Export, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif

		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif

		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* class ID, event ID, aete ID, uniqueString */
		
		SupportedModes
			{
			noBitmap, doesSupportGrayScale,
			doesSupportIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
			},
			
		EnableInfo { "true" }

	}
};

resource 'PiMI' (ResourceID, purgeable)
{ 
	  latestExportVersion,
	  latestExportSubVersion,
	  0,
	  supportsGrayScale +
	  supportsIndexedColor +
	  supportsRGBColor +
	  supportsCMYKColor +
	  supportsHSLColor +
	  supportsHSBColor +
	  supportsMultichannel +
	  supportsDuotone +
	  supportsLABColor,
	  '    ', /* No required host */
	  {},
};

/* Prompt strings */

resource StringResource (kNoHistories, "No histories", purgeable)
{
	"No history resources in this document."
};

resource StringResource (kHistStatus, "History status", purgeable)
{
	"^0 to ^1 of ^2"
};


/* About resources */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1990-6, Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"An example plug-in export module for Adobe Photoshop."
};

/* UI */

resource 'DLOG' (ResourceID+1, "UI", purgeable)
{
	{133, 229, 381, 625},
	movableDBoxProc,
	visible,
	noGoAway,
	0x0,
	ResourceID+1,
	plugInName
};

resource 'DITL' (ResourceID+1, "UI", purgeable)
{
	{	/* array DITLarray: 14 elements */
		/* [1] */
		{12, 298, 32, 378},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{42, 298, 62, 378},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{81, 298, 101, 378},
		Button {
			enabled,
			"Trim First"
		},
		/* [4] */
		{107, 298, 127, 378},
		Button {
			enabled,
			"Trim Last"
		},
		/* [5] */
		{145, 298, 165, 378},
		Button {
			enabled,
			"Up"
		},
		/* [6] */
		{171, 298, 191, 378},
		Button {
			enabled,
			"Down"
		},
		/* [7] */
		{204, 288, 244, 392},
		StaticText {
			disabled,
			""
		},
		/* [8] */
		{1, 2, 36, 284},
		StaticText {
			disabled,
			""
		},
		/* [9] */
		{35, 2, 70, 284},
		StaticText {
			disabled,
			""
		},
		/* [10] */
		{69, 2, 104, 284},
		StaticText {
			disabled,
			""
		},
		/* [11] */
		{103, 2, 138, 284},
		StaticText {
			disabled,
			""
		},
		/* [12] */
		{137, 2, 172, 284},
		StaticText {
			disabled,
			""
		},
		/* [13] */
		{171, 2, 206, 284},
		StaticText {
			disabled,
			""
		},
		/* [14] */
		{205, 2, 240, 284},
		StaticText {
			disabled,
			""
		}
	}
};

/* scripting dictionary resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		vendorName,											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{},													/* structure for filters */
		{													/* non-filter plug-in class here */
			vendorName " history",							/* unique class name */
			ourClassID,										/* class ID, must be unique or Suite ID */
			"outbound plug-in",								/* optional description */
			{												/* define inheritance */
				"<Inheritance>",							/* must be exactly this */
				keyInherits,								/* must be keyInherits */
				classExport,								/* parent: Format, Import, Export */
				"parent class export",						/* optional description */
				flagsSingleProperty,						/* if properties, list below */
				
				"in",										/* our path */
				keyIn,										/* common key */
				typePlatformFilePath,						/* correct path for platform */
				"file path",								/* optional description */
				flagsSingleProperty
				
				/* no more properties */
			},
			{}, /* elements (not supported) */
			/* class descriptions */
		},
		{}, /* comparison ops (not supported) */
		{}	/* any enumerations */
	}
};