/*
	File: IllustratorExport.c

	Copyright (c) 1993-6, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Paths to Illustrator example.
*/


#if __MWERKS__
#include <SetupA4.h> // A4-globals
#include <A4Stuff.h> // A4-globals
#endif

#if defined(THINK_C) || defined(__MWERKS__)
#define ENTRYPOINT main
#endif

#include "IllustratorExport.h"

Handle hDllInstance = NULL;

const int32 kSizeSavedPath = sizeof(SavedPathElement);

/*****************************************************************************/

void InitGlobals (GPtr globals);
void DoPrepare (GPtr globals);
void DoStart (GPtr globals);
void DoContinue (GPtr globals);
void DoFinish (GPtr globals);

Boolean WriteResString (GPtr globals, short resID);
Boolean WriteReplaceString (GPtr globals, short resID, Str255 s2);
Boolean CheckRange (GPtr globals, int32 scalar, int32 pixels);
Boolean ConvertSavedPoint (GPtr globals,
						  int32 savedX,
						  int32 savedY,
						  Fixed *x,
						  Fixed *y);
Boolean WriteToFile (GPtr globals, void *data, long count);
Boolean WriteString (GPtr globals, Str255 s);
Boolean WriteChar (GPtr globals, unsigned char c);
Boolean WriteNewLine (GPtr globals);
Boolean WriteSpace (GPtr globals);
Boolean WriteInteger (GPtr globals, short x);
void ConvertFixed (Fixed value, int16 places, Boolean trim, Str255 s);
Boolean WriteFixed (GPtr globals, Fixed x, int16 places);
Boolean WriteBoundingBox (GPtr globals);
Boolean WriteFileHeader (GPtr globals);
Boolean WritePoint (GPtr globals, Fixed x, Fixed y);
Boolean WriteMoveTo (GPtr globals, Fixed x, Fixed y);
Boolean WriteLineTo (GPtr globals, Fixed x, Fixed y);
Boolean WriteSegment (GPtr globals,
					 Fixed startX,
					 Fixed startY,
					 Fixed fwdX,
					 Fixed fwdY,
					 Fixed bwdX,
					 Fixed bwdY,
					 Fixed endX,
					 Fixed endY,
					 Boolean cusp);
Boolean WriteNewPath (GPtr globals, Boolean closeOldPath);
void AdvanceOneElement (SavedPathElement **path, int32 *elements);
void SkipUnidentifiedElements (SavedPathElement **path, int32 *elements);
Boolean GetNextElement (SavedPathElement **path,
					   int32 *elements,
					   int16 *kind);
Boolean WriteSubPath (GPtr globals,
					 SavedPathElement **path, 
					 int32 *elements);
Boolean WritePath (GPtr globals, Handle savedPath);
Handle GetIndexedPath (GPtr globals, int32 index);
Boolean WriteIndexedPath (GPtr globals, int32 index);
Boolean WriteAllPaths (GPtr globals);
Boolean WriteFileTrailer (GPtr globals);
Boolean ConvertToPoints (GPtr globals,
						int16 pixels,
						Fixed resolution,
						Fixed *value);
Boolean CalculateBoundingBox (GPtr globals);
Boolean WriteBody (GPtr globals);
Fixed DoFixMul (int32 x, int32 y);
int32 DoFloor (double x);
Handle FixupPath(GPtr globals, Handle path, int howMany);
void FixupOne( char *data, SavedPathElement **path );
unsigned32 SwapLong( char *data );
unsigned16 SwapWord( char *data );

/*****************************************************************************/

/* All calls to the plug-in module come through this routine. It must be
   placed first in the resource. To achieve this, most development systems
   require that this be the first routine in the source. */

#if MSWindows
	   void ENTRYPOINT (short selector,
						ExportRecord *exportParamBlock,
						long *data,
						short *result)
#else
pascal void ENTRYPOINT (short selector,
						ExportRecord *exportParamBlock,
						long *data,
						short *result)
#endif
	{
	
	Globals globalValues;
	GPtr globals = &globalValues;
	
	#if __MWERKS__
	EnterCodeResource(); // A4-globals
	#endif	
	
	if (!*data)
		{
		
		InitGlobals (globals);

		*data = (long) NewHandle (sizeof (Globals));
		
		if (!*data)
			{
			*result = memFullErr;
			return;
			}
			
		** (GHdl) *data = globalValues;
		
		}
		
	globalValues = ** (GHdl) *data;
		
	gStuff = exportParamBlock;
	gResult = noErr;
		
	switch (selector)
		{
		
		case exportSelectorAbout:
			DoAbout (globals);
			break;
			
		case exportSelectorPrepare:
			DoPrepare (globals);
			break;
		
		case exportSelectorStart:
			DoStart (globals);
			break;
		
		case exportSelectorContinue:
			DoContinue (globals);
			break;
		
		case exportSelectorFinish:
			DoFinish (globals);
			break;
			
		default:
			gResult = exportBadParameters;
		
		}
		
	*result = gResult;
	** (GHdl) *data = globalValues;

	#if __MWERKS__
	ExitCodeResource(); // A4-globals
	#endif
		
	}

/*****************************************************************************/

void InitGlobals (GPtr globals)
{
	gPopUpValue = 0;
}

/*****************************************************************************/

void ValidateParameters (GPtr globals)
{ // check for parameters and initial values before we use them
	gQueryForParameters = true; // assume yes
	gAliasHandle = NULL; // no handle, yet
	if (gPopUpValue < 1) gPopUpValue = 0;
}
		

/*****************************************************************************/

/* We don't particularly need any memory. */

void DoPrepare (GPtr globals)
{
	
	gStuff->maxData = 0;
	
}

/*****************************************************************************/

/* Display the point-out-of range error message. */

Boolean CheckRange (GPtr globals, int32 scalar, int32 pixels)
{
		
	if (scalar < 0) scalar = -scalar;
	
	scalar = scalar >> 24;
	
	pixels = pixels >> 16;
	
	if (scalar * pixels >= 0x00008000L) // magic number
	{
		DisplayStopAlert (globals, kAIRange);
		return FALSE;
	}
		
	return TRUE;
	
}

/*****************************************************************************/

Fixed DoFixMul (int32 x, int32 y)
{
	double xd = Fixed16ToDouble(x);
	double yd = Fixed16ToDouble(y);
	double pd = xd * yd;
	return DoubleToFixed16(pd);
}

/*****************************************************************************/

Boolean ConvertSavedPoint (GPtr globals,
						  int32 savedX,
						  int32 savedY,
						  Fixed *x,
						  Fixed *y)
{
	
	if (CheckRange (globals, savedX, gDocWidth) &&
		CheckRange (globals, savedY, gDocHeight))
		{
		
		*x = DoFixMul (savedX, gDocWidth >> 8);
		*y = DoFixMul (kSavedOne - savedY, gDocHeight >> 8);
		
		return TRUE;
		
		}
		
	else
		return FALSE;
	
}

/*****************************************************************************/

void AppendStringAndString (Str255 s, Str255 append)
{	
	AppendString(s, append, 1, append[0]);
}

/*****************************************************************************/

void AppendStringAndCharacter (Str255 s, unsigned char c)
{	
	AppendCharToString(s, c);
}

/*****************************************************************************/

Boolean WriteToFile (GPtr globals, void *data, long count)
	{
	
	if (gResult == noErr)
		gResult = FSWrite (gFRefNum, &count, data);
	
	return (gResult == noErr);
	
	}

/*****************************************************************************/

Boolean WriteString (GPtr globals, Str255 s)
	{
	
	if (s[0] == 0)
		return TRUE;
	
	return WriteToFile (globals, &(s[1]), s[0]);
	
	}

/*****************************************************************************/

Boolean WriteChar (GPtr globals, unsigned char c)
	{
	
	return WriteToFile (globals, &c, 1);
	
	}

/*****************************************************************************/

Boolean WriteNewLine (GPtr globals)
	{
	
	return WriteChar (globals, 0x0D);
	
	}
	
/*****************************************************************************/

Boolean WriteSpace (GPtr globals)
	{
	
	return WriteChar (globals, 0x20);
	
	}
	
/*****************************************************************************/

Boolean WriteResString (GPtr globals, short resID)
{
	
	Str255 	s = "";
	
	PIGetString(s, (int32)resID);
	return WriteString (globals, s);
}

/*****************************************************************************/

Boolean WriteReplaceString (GPtr globals, short resID, Str255 s2)
{
	
	Str255 	s = "";

	PIGetString(s, (int32)resID);
	PIParamText(s, s2, NULL, NULL); // replaces ^0 with s2
	
	return WriteString (globals, s);
}

/*****************************************************************************/

Boolean WriteInteger (GPtr globals, short x)
	{
	
	Str255 s;
	
	NumToString (x, s);
	
	return WriteString (globals, s);
	
	}

/*****************************************************************************/

void ConvertFixed (Fixed value, int16 places, Boolean trim, Str255 s)
{
	double x = 0;
	long l = 0;

	if (places > 0)
	{ // use decimal converter
		x = PIFixedToDouble(value, kHard16);
		DoubleToString (x, s, places);
	}
	else
	{ // no decimal number required
		l = fixed2long(value);
		NumToString(l, s);
	}
}
/*****************************************************************************/

Boolean WriteFixed (GPtr globals, Fixed x, int16 places)
{
	
	Str255 s;
	
	ConvertFixed (x, places, TRUE, s);
	
	return WriteString (globals, s);
	
}

/*****************************************************************************/

Boolean WriteBoundingBox (GPtr globals)
{
	

	return WriteResString	(globals, kPSBoundingBox)		&&
		   WriteFixed 		(globals, 0, 0) 				&&
		   WriteSpace		(globals)						&&
		   WriteFixed		(globals, 0, 0)					&&
		   WriteSpace 		(globals) 						&&
		   WriteFixed 		(globals, gDocWidth, 0) 		&&
		   WriteSpace 		(globals)						&&
		   WriteFixed 		(globals, gDocHeight, 0) 		&&
		   WriteNewLine 	(globals) 						&&

	       WriteResString	(globals, kPSHiResBoundingBox) 	&&
		   WriteFixed 		(globals, 0, 4) 				&&
		   WriteSpace		(globals)						&&
		   WriteFixed		(globals, 0, 4)					&&
		   WriteSpace 		(globals) 						&&
		   WriteFixed 		(globals, gDocWidth, 4) 		&&
		   WriteSpace 		(globals)						&&
		   WriteFixed 		(globals, gDocHeight, 4) 		&&
		   WriteNewLine 	(globals) 						&&
	   
	       WriteResString	(globals, kPSCropmarks) 		&&
		   WriteFixed 		(globals, 0, 0) 				&&
		   WriteSpace		(globals)						&&
		   WriteFixed		(globals, 0, 0)					&&
		   WriteSpace 		(globals) 						&&
		   WriteFixed 		(globals, gDocWidth, 0) 		&&
		   WriteSpace 		(globals)						&&
		   WriteFixed 		(globals, gDocHeight, 0) 		&&
		   WriteNewLine 	(globals);

#if 0 // old bounding box		   
	return WriteIndexedString (globals, kStringsID, strBoundingBox) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, gDocWidth, 0) &&
		   WriteSpace (globals),
		   WriteFixed (globals, gDocHeight, 0) &&
		   WriteNewLine (globals) &&
		   
	       WriteIndexedString (globals, kStringsID, strHiResBoundingBox) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, gDocWidth, 4) &&
		   WriteSpace (globals),
		   WriteFixed (globals, gDocHeight, 4) &&
		   WriteNewLine (globals) &&
		   
	       WriteIndexedString (globals, kStringsID, strCropmarks) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, 0, 0) &&
		   WriteSpace (globals) &&
		   WriteFixed (globals, gDocWidth, 4) &&
		   WriteSpace (globals),
		   WriteFixed (globals, gDocHeight, 4) &&
		   WriteNewLine (globals);
#endif // old bounding box
		   
}

/*****************************************************************************/

Boolean WriteTitle (GPtr globals)
{
	Str255 s = "";

	SmallFileName(s, gStuff->filename); // returns small file name as pascal
	return WriteReplaceString(globals, kPSTitle, s);
}

/*****************************************************************************/

Boolean WriteFileHeader (GPtr globals)
{
	
	return WriteResString		(globals, kPSHeader)		&&
		   WriteTitle			(globals)					&&
		   WriteResString		(globals, kPSProcSets)		&&
		   WriteBoundingBox		(globals)					&&
		   WriteResString		(globals, kPSBegin);
}

/*****************************************************************************/

Boolean WritePoint (GPtr globals, Fixed x, Fixed y)
{
	
	return WriteFixed		(globals, x, 4) 		&&
		   WriteSpace		(globals)		 		&&
		   WriteFixed 		(globals, y, 4) 		&&
		   WriteSpace 		(globals);
	
}

/*****************************************************************************/

Boolean WriteMoveTo (GPtr globals, Fixed x, Fixed y)
{
	
	return WritePoint 		(globals, x, y) 		&&
           WriteChar 		(globals, 'm') 			&&
		   WriteNewLine 	(globals);
	
}

/*****************************************************************************/

Boolean WriteLineTo (GPtr globals, Fixed x, Fixed y)
	{
	
	return WritePoint 		(globals, x, y) 		&&
           WriteChar 		(globals, 'l') 			&&
		   WriteNewLine 	(globals);
	
	}

/*****************************************************************************/

Boolean WriteSegment (GPtr globals,
					 Fixed startX,
					 Fixed startY,
					 Fixed fwdX,
					 Fixed fwdY,
					 Fixed bwdX,
					 Fixed bwdY,
					 Fixed endX,
					 Fixed endY,
					 Boolean cusp)
{
	
	Boolean startTangent = (startX != fwdX) || (startY != fwdY);
	Boolean endTangent   = (endX   != bwdX) || (endY   != bwdY);
	
	Boolean result = TRUE;
	
	unsigned char cmd;
	
	if (startTangent)
		result = result && WritePoint (globals, fwdX, fwdY);
		
	if (endTangent)
		result = result && WritePoint (globals, bwdX, bwdY);
		
	result = result && WritePoint (globals, endX, endY);
	
	if (startTangent && endTangent)
		cmd = 'C';
	else if (startTangent)
		cmd = 'Y';
	else if (endTangent)
		cmd = 'V';
	else
		cmd = 'L';
		
	if (!cusp)
		cmd += 'a' - 'A';
		
	return result && WriteChar (globals, cmd) && WriteNewLine (globals);
	
}

/*****************************************************************************/

Boolean WriteNewPath (GPtr globals, Boolean closeOldPath)
{
	
	return	WriteChar 	(globals, (char)(closeOldPath ? 'n' : 'N')) 	&&
			WriteNewLine	(globals);
}

/*****************************************************************************/

void AdvanceOneElement (SavedPathElement **path, int32 *elements)
{
	
	if (*elements > 1)
		{
		--(*elements);
		++(*path);
		}
	
	else
		{
		*path = NULL;
		*elements = 0;
		}
	
}

/*****************************************************************************/

void SkipUnidentifiedElements (SavedPathElement **path, int32 *elements)
{
	
	while (*path != NULL && *elements > 0)
		{
		
		int16 kind = (**path).kind;
		
		if (kMinPathKind <= kind && kind <= kMaxPathKind)
			return;
			
		AdvanceOneElement (path, elements);
		
		}
	
}

/*****************************************************************************/

Boolean GetNextElement (SavedPathElement **path,
					   int32 *elements,
					   int16 *kind)
{
	
	AdvanceOneElement (path, elements);
	
	SkipUnidentifiedElements (path, elements);
	
	if (*path != NULL && *elements > 0)
		{
		*kind = (**path).kind;
		return TRUE;
		}
	
	return FALSE;
	
}

/*****************************************************************************/

Boolean WriteSubPath (GPtr globals,
					 SavedPathElement **path, 
					 int32 *elements)
{
	
	int16 kind;
	Boolean closed;
	
	Fixed finalBwdX;
	Fixed finalBwdY;
	Fixed initialX;
	Fixed initialY;
	Fixed prevX;
	Fixed prevY;
	Fixed fwdX;
	Fixed fwdY;
	Fixed bwdX;
	Fixed bwdY;
	Fixed curX;
	Fixed curY;
	Boolean finalCusp;
	
	int32 length;
	
	int16 cuspKind;
	int16 smoothKind;
	char z = 0;
	
	SkipUnidentifiedElements (path, elements);
	
	if (*path == NULL || *elements == 0)
		return TRUE;
		
	/* Process the subpath header. */
		
	kind = (**path).kind;
	length = ((**path).value [0]) >> 16;
	
	if ((kind != kOpenSubpathLength && kind != kClosedSubpathLength) ||
		length <= 0)
		{
		DisplayStopAlert (globals, kAIInvalidPath);
		return FALSE;
		}
		
	closed = (kind == kClosedSubpathLength);
	
	if (closed)
		{
		cuspKind = kCuspClosedKnot;
		smoothKind = kSmoothClosedKnot;
		}
	else
		{
		cuspKind = kCuspOpenKnot;
		smoothKind = kSmoothOpenKnot;
		}
		
	/* Process the first knot. */
	
	if (!GetNextElement (path, elements, &kind) ||
		(kind != cuspKind && kind != smoothKind))
		{
		DisplayStopAlert (globals, kAIInvalidPath);
		return FALSE;
		}
		
	if (!ConvertSavedPoint (globals,
							(**path).value [3],
							(**path).value [2],
							&initialX,
							&initialY) ||
		!ConvertSavedPoint (globals,
							(**path).value [1],
							(**path).value [0],
							&finalBwdX,
							&finalBwdY) ||
		!ConvertSavedPoint (globals,
							(**path).value [5],
							(**path).value [4],
							&fwdX,
							&fwdY))
		return FALSE;

	finalCusp = (kind == cuspKind);
		
	if (!WriteMoveTo (globals, initialX, initialY))
		return FALSE;
		
	prevX = initialX;
	prevY = initialY;
		
	while (--length > 0)
		{
		
		if (!GetNextElement (path, elements, &kind) ||
			(kind != cuspKind && kind != smoothKind))
			{
			DisplayStopAlert (globals, kAIInvalidPath);
			return FALSE;
			}
			
		if (!ConvertSavedPoint (globals,
								(**path).value [1],
								(**path).value [0],
								&bwdX,
								&bwdY) ||
			!ConvertSavedPoint (globals,
								(**path).value [3],
								(**path).value [2],
								&curX,
								&curY) ||
			!WriteSegment (globals,
						   prevX,
						   prevY,
						   fwdX,
						   fwdY,
						   bwdX,
						   bwdY,
						   curX,
						   curY,
						   (Boolean)(kind == cuspKind)) ||
			!ConvertSavedPoint (globals,
								(**path).value [5],
								(**path).value [4],
								&fwdX,
								&fwdY))
			return FALSE;
			
		prevX = curX;
		prevY = curY;
		
		}
		
	if (closed && !WriteSegment (globals,
								 prevX,
								 prevY,
								 fwdX,
								 fwdY,
								 finalBwdX,
								 finalBwdY,
								 initialX,
								 initialY,
								 finalCusp))
		return FALSE;
		
	return WriteNewPath (globals, closed);
	
}
							 
/*****************************************************************************/

Boolean WritePath (GPtr globals, Handle savedPath)
{
	
	int32 elements = PIGetHandleSize(savedPath) / kSizeSavedPath;
	SavedPathElement *path;
	
	Boolean notdone = TRUE;
	
	if ( PIGetHandleSize(savedPath) < (int32)( elements * kSizeSavedPath ) )
	{
		DisplayStopAlert (globals, kAIInvalidPath);
		return FALSE;
	}
		
	PILockHandle (savedPath, FALSE);
	
	path = (SavedPathElement *) *savedPath;

	while (notdone && path != NULL && elements > 0)
		{
		notdone = WriteSubPath (globals, &path, &elements);
		AdvanceOneElement (&path, &elements);
		}
		
	PIUnlockHandle (savedPath);
		
	return notdone;
	
}

/*****************************************************************************/

unsigned16 SwapWord( char *data )
{
	unsigned16 tmp = 0;
	short loop;

	// To swap the bytes, do: #if MSWindows for (loop = 1; loop >= 0; loop--)
	// But, for some reason, this seems to fix everything for us
	for (loop = 0; loop < 2; loop++)
	{
		tmp <<= 8;
		tmp |= (unsigned8)data[loop];
	}
	return tmp;
}

/*****************************************************************************/

unsigned32 SwapLong( char *data )
{
	unsigned32 tmp = 0;
	short loop;

	// to swap the bytes, do: #if MSWindows for (loop = 3; loop >= 0; loop--)
	// For some reason, don't need to do it, cuz this works
	for (loop = 0; loop < 4; loop++)
	{
		tmp <<= 8;
		tmp |= (unsigned8)data[loop];
	}
	return tmp;
}

/*****************************************************************************/

void FixupOne( char *data, SavedPathElement **path )
{
	short x = 0;

	(**path).kind = SwapWord( &data[0] );
	
	for (x = 0; x < 6; x++)
		(**path).value[x] = SwapLong( &data[2 + (x * 4)] );
	
}

/*****************************************************************************/

Handle FixupPath(GPtr globals, Handle path, int howMany )
{
	short x = 0;

	Handle h = PINewHandle(howMany * kSizeSavedPath);

	if (h != NULL)
	{ // got our new handle.  Go to town with it.

		char *data;
		SavedPathElement *newpath;

		data = *path;
		newpath = (SavedPathElement *) *h;

		while (howMany)
		{
			FixupOne( data, &newpath );
			
			howMany--;
			data += kHardPathLength;
			++newpath;
		}
		PIDisposeHandle( path ); // done with this
	}
	return h;
}

/*****************************************************************************/

Handle GetIndexedPath (GPtr globals, int32 index)
{
	
	Handle pathData;
	
	if (gResult != noErr)
		return NULL;
	
	gResult = GetComplex(propPathContents, index, &pathData);
	
	if (gResult == noErr)
		return FixupPath(globals, pathData, 
				(PIGetHandleSize( pathData ) / kHardPathLength) );
	else
		return NULL;

}

/*****************************************************************************/

Boolean WriteIndexedPath (GPtr globals, int32 index)
{
	
	Handle pathData = GetIndexedPath (globals, index);
	
	if (pathData != NULL)
		{
		Boolean result = WritePath (globals, pathData);
		/* When we get a handle back from the property routines, it is our
		   responsibility to dispose of it. */
		PIDisposeHandle (pathData);
		return result;
		}
	return FALSE;
}

/*****************************************************************************/

Boolean GetPathCount (GPtr globals, int32 *count)
{
	
	if (gResult != noErr)
		return FALSE;
	
	gResult = GetSimple(propNumberOfPaths, count);

	if (gResult == noErr)
		return TRUE;
		
	*count = 0;
	
	return FALSE;

}

/*****************************************************************************/

int16 GetPathIndex (GPtr globals, OSType pathProperty)
{
	
	int32 	simple;
	OSErr	result = noErr;
	
	if (gResult != noErr) return -1;

	result = GetSimple(pathProperty, &simple);

	if (result != noErr)
		return -1;
		
	return (int16) simple;
	
	}

/*****************************************************************************/

Boolean GetPathName (GPtr globals, int32 index, Str255 s)
{
	
	Handle complex;

	gResult = GetComplex(propPathName, index, &complex);
	
	if (gResult != noErr)
		return FALSE;
		
	if (complex == NULL)
		{
		gResult = errPlugInHostInsufficient;
		return FALSE;
		}
		
	PIHandle2String(complex, s);

	// We were given this handle.  It's our job to dispose it.
	PIDisposeHandle (complex);
	
	return TRUE;
	
}

/*****************************************************************************/

Boolean WriteAllPaths (GPtr globals)
	{
	
	int32 pathCount;
	int32 index;
	
	if (!GetPathCount (globals, &pathCount))
		return FALSE;
		
	for (index = 0; index < pathCount; ++index)
		{
		
		UpdateProgress (index, pathCount);
		
		if (TestAbort ())
			{
			gResult = userCanceledErr; // exit gracefully
			return FALSE;
			}
			
		if (!WriteIndexedPath (globals, index))
			return FALSE;
			
		}
			
	return TRUE;
	
	}

/*****************************************************************************/

Boolean WriteFileTrailer (GPtr globals)
{
	return WriteResString (globals, kPSTrailer);
}

/*****************************************************************************/

int32 DoFloor (double x)
{
	int32 retn = 0;

	if (x < 0)
		retn = (int32)(x-1); // floor -3.8 -> -4, etc.
	else
		retn = (int32)(x); // floor 2.9 -> 2, etc.

	return retn;
}

/*****************************************************************************/

Boolean ConvertToPoints (GPtr globals,
						int16 pixels,
						Fixed resolution,
						Fixed *value)
	{
	
	double xValue;
	
	if (pixels * 72.0 / (resolution / 65536.0) > 32767.0)
		{
		DisplayStopAlert (globals, kAITooBig);
		return FALSE;
		}
		
	xValue = pixels * (72.0 / (resolution / 65536.0));
		
	*value = (Fixed) (int32) DoFloor (xValue * 65536.0 + 0.5);
	
	return TRUE;
	}

/*****************************************************************************/

Boolean CalculateBoundingBox (GPtr globals)
	{
	
	return ConvertToPoints (globals,
							gStuff->imageSize.v,
							gStuff->imageVRes,
							&gDocHeight) &&
		   ConvertToPoints (globals,
							gStuff->imageSize.h,
							gStuff->imageHRes,
							&gDocWidth);
	
	}

/*****************************************************************************/

Boolean WriteBody (GPtr globals)
	{
	
	if (gPopUpValue == iDocBounds)
		return TRUE;
	else if (gPopUpValue == iAllPaths)
		return WriteAllPaths (globals);
	else if (gPopUpValue >= iPathStart)
		return WriteIndexedPath (globals, gPopUpValue - iPathStart);
	else
		return FALSE;
	
	}
	
/*****************************************************************************/

/* Requests pointer to the first part of the image to be filtered. */

void DoStart (GPtr globals)
{
	
	PISetRect (&gStuff->theRect, 0, 0, 0, 0);
	gStuff->hiPlane = 0;
	gStuff->loPlane = 0;
	
	if (!WarnPropertyAvailable())
	{ 
		gResult = errPlugInHostInsufficient; // exit w/error
		return;
	}
		
	ValidateParameters (globals);
	/* if stuff hasn't been initialized that we need, do it,
		then go check if we've got scripting commands to
		override our settings */

	ReadScriptParams (globals);
	
	(void) (CalculateBoundingBox (globals) &&
		    PromptForFile (globals) &&
			CreateExportFile (globals) &&
		    WriteFileHeader (globals) &&
		    WriteBody (globals) &&
		    WriteFileTrailer (globals));
	
	CloseExportFile (globals);
	// always do this	
}

/*****************************************************************************/

// Filters the area and requests the next area. We never call this since we
// do all our work in DoStart.  But just in case, we'll reset everything.

void DoContinue (GPtr globals)
{
	PISetRect (&gStuff->theRect, 0, 0, 0, 0);
	gResult = userCanceledErr; // exit gracefully
}

/*****************************************************************************/

/* This routine will always be called if DoStart does not return an error
   (even if DoContinue returns an error or the user aborts the operation).
   This allows the module to perform any needed cleanup.  None is required
   in this example. */

void DoFinish (GPtr globals)
{
	WriteScriptParams (globals);
}

/*****************************************************************************/