/*
	File: IllustratorExport.h

	Copyright (c) 1993-6, Adobe Systems Incorporated.
	All rights reserved.

	Header file for Illustrator Export example.
*/

#ifndef __IllustratorExport_H__
#define __IllustratorExport_H__

#include "PIExport.h"
#include "PIProperties.h"

#if Macintosh
#include "FileUtilities.h"
#else
#include "WinFileUtils.h"
#endif

//#include <Math.h>

/*****************************************************************************/

#define keyOurPath			'patH'

// Prompt and Postscript strings
#define kPrompt				16100
#define kCreatorAndType		kPrompt+1
#define kSuffix				kCreatorAndType+1
#define kPSTitle			kSuffix+1
#define kPSBoundingBox		kPSTitle+1
#define kPSHiResBoundingBox	kPSBoundingBox+1
#define kPSCropmarks		kPSHiResBoundingBox+1
#define kPSHeader			kPSCropmarks+1
#define kPSProcSets			kPSHeader+1
#define kPSBegin			kPSProcSets+1
#define kPSTrailer			kPSBegin+1
#define kDocBounds			kPSTrailer+1
#define kAllPaths			kDocBounds+1

// Alerts
#define kAINeed30			16200
#define kAITooBig			kAINeed30+1
#define kAIRange			kAITooBig+1
#define kAIInvalidPath		kAIRange+1

// Dialog items
#define dPopUp				13
		
// Pop up menu items
#define iDocBounds			1
#if MSWindows
#define iAllPaths			2
#define iPathStart			3
#else // we can do separators, so use them
#define iAllPaths			3
#define iPathStart			5
#endif


// Conversion defines
#define kSavedOne 0x01000000L
	
/*****************************************************************************/
// Path types

enum
{
	kClosedSubpathLength,
	kSmoothClosedKnot,
	kCuspClosedKnot,
	kOpenSubpathLength,
	kSmoothOpenKnot,
	kCuspOpenKnot
};

#define kMinPathKind kClosedSubpathLength
#define kMaxPathKind kCuspOpenKnot

/*****************************************************************************/

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

typedef struct SavedPathElement
{
	int16	kind;
	int32	value [6];
} SavedPathElement;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#define kHardPathLength 26 // absolute size for a path

/*****************************************************************************/

extern Handle hDllInstance;

typedef struct Globals
{

	short result;
	ExportRecord *exportParamBlock;
	Boolean queryForParameters;
	Boolean sameNames;
	short fRefNum;	
	short popUpValue;
	
	Fixed docHeight;
	Fixed docWidth;
	
	#if Macintosh
	AliasHandle	aliasHandle;
	#else
	Handle	aliasHandle;
	#endif

} Globals, *GPtr, **GHdl;

#define gResult 		(globals->result)
#define gStuff  		(globals->exportParamBlock)

#define gDocHeight 		(globals->docHeight)
#define gDocWidth		(globals->docWidth)

#define gFRefNum 		(globals->fRefNum)
#define gAliasHandle	(globals->aliasHandle)
#define gSameNames		(globals->sameNames)

#define gPopUpValue 	(globals->popUpValue)

#define gQueryForParameters	(globals->queryForParameters)

/*****************************************************************************/

void DoAbout (GPtr globals);
void DisplayStopAlert (GPtr globals, short id);
Boolean WriteTitle (GPtr globals);
Boolean PromptForFile (GPtr globals);
void SmallFileName(Str255 s, const Str255 file);
void AppendStringAndString (Str255 s, Str255 append);
void AppendStringAndCharacter (Str255 s, unsigned char c);
Boolean GetPathCount (GPtr globals, int32 *count);
int16 GetPathIndex (GPtr globals, OSType pathProperty);
Boolean GetPathName (GPtr globals, int32 index, Str255 s);
Boolean	ReadScriptParams (GPtr globals);
OSErr	WriteScriptParams (GPtr globals);

Boolean QueryForExportFile (GPtr globals);
Boolean CreateExportFile (GPtr globals);
Boolean WriteExportFile (GPtr globals);
Boolean CloseExportFile (GPtr globals);
Handle PopUp2Handle (GPtr globals, short popup);
short Handle2PopUp (GPtr globals, Handle h);
void ValidateParameters (GPtr globals);

/*****************************************************************************/

#endif /* __IllustratorExport_H__ */