/*
	File: IllustratorExport.r
	by Mark Hamburg

	Copyright (c) 1990-96, Adobe Systems Incorporated.
	All rights reserved.
	
	Rez source file for Paths To Illustrator module.
*/

/********************************************************************************/
/* Defines needed by included files */

#define plugInName			"IllustratorExport"
#define VersionString		"5.0"

/********************************************************************************/

#if Macintosh
#include "Types.r"
#include "SysTypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/********************************************************************************/

#define plugInComment 		"Illustrator export plug-in"
#define vendorName			"AdobeSDK"

#define ourSuiteID			'sdKB'
#define ourClassID			'illE'
#define ourEventID			typeNull // must be this

#define keyOurPath			'patH'

// Prompt and Postscript strings
#define kPrompt				16100
#define kCreatorAndType		kPrompt+1
#define kSuffix				kCreatorAndType+1
#define kPSTitle			kSuffix+1
#define kPSBoundingBox		kPSTitle+1
#define kPSHiResBoundingBox	kPSBoundingBox+1
#define kPSCropmarks		kPSHiResBoundingBox+1
#define kPSHeader			kPSCropmarks+1
#define kPSProcSets			kPSHeader+1
#define kPSBegin			kPSProcSets+1
#define kPSTrailer			kPSBegin+1
#define kDocBounds			kPSTrailer+1
#define kAllPaths			kDocBounds+1

// Alerts
#define kAINeed30			16200
#define kAITooBig			kAINeed30+1
#define kAIRange			kAITooBig+1
#define kAIInvalidPath		kAIRange+1

/********************************************************************************/

resource 'PiPL' (ResourceID, purgeable)
{
	{
		Kind { Export },
		Name { "IllustratorExport..." },
		Version { (latestExportVersion << 16) | latestExportSubVersion },
		
		#if Macintosh
		Code68K { Export, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif
		
		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif

		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* class ID, event ID, aete ID, uniqueString */
		
		SupportedModes
		{
			noBitmap, doesSupportGrayScale,
			doesSupportIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
		},
		
		EnableInfo { "in (PSHOP_ImageMode, GrayScaleMode,"
						 "IndexedColorMode, RGBMode,"
						 "CMYKMode, HSLMode,"
					     "HSBMode, MultichannelMode,"
					     "DuotoneMode, LabMode)" }				
	}
};


resource 'PiMI' (ResourceID, purgeable)
	{ latestExportVersion,
	  latestExportSubVersion,
	  0,
	  supportsGrayScale +
	  	supportsRGBColor +
		supportsCMYKColor +
		supportsHSLColor +
		supportsHSBColor +
		supportsMultichannel +
		supportsDuotone +
		supportsLABColor,
	  '    ', /* No required host */
	  {}
	 };

/* About resources */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1991-6, Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"A plug-in module to export pen paths to Adobe Illustrator."
};

/* String resources
 *
 * I'm using StringResource ('STR ') resources, which are single-string
 * resources, because they are converted to Windows by CNVTPIPL.
 *
 * 'STR#' (multiple, indexed string) resources are not recognized and have
 * no intuitive mapping to Windows.
 */

resource StringResource (kPrompt, "Prompt", purgeable)
{
	"Export paths to file:"
};

resource StringResource (kCreatorAndType, "CreatorAndType", purgeable)
{
	"ART5TEXT"
}; // first 4 = creator; second 4 = type

resource StringResource (kSuffix, "Suffix", purgeable)
{
	".ai"
};

resource StringResource (kPSTitle, "PSTitle", purgeable)
{
	"%%Title: (^0)\n"
};

resource StringResource (kPSBoundingBox, "BoundingBox", purgeable)
{
	"%%BoundingBox: "
};

resource StringResource (kPSHiResBoundingBox, "HiResBoundingBox", purgeable)
{
	"%%HiResBoundingBox: "
};

resource StringResource (kPSCropmarks, "Cropmarks", purgeable)
{
	"%AI3_Cropmarks: "
};

resource StringResource (kPSHeader, "Header", purgeable)
{
	"%!PS-Adobe-2.0 EPSF-1.2\n"
	"%%Creator: Adobe Photoshop IllustratorExport\n"
};

resource StringResource (kPSProcSets, "ProcSets", purgeable)
{
	"%%DocumentProcSets: Adobe_Illustrator_1.1 0 0\n"
	"%%ColorUsage: Black&White\n"
};

resource StringResource (kPSBegin, "PSBegin", purgeable)
{
	"%%DocumentPreview: None\n"
	"%%EndComments\n"
	"%%EndProlog\n"
	"%%BeginSetup\n"
	"About_Illustrator_1.1 begin\n"
	"n\n"
	"%%EndSetup\n"
	"0.0 0.0 0.0 1.0 k\n"
	"0 i 0 J 0 j 1 w 4 M []0 d\n"
	"%%Note:\n"
};

resource StringResource (kPSTrailer, "PSTrailer", purgeable)
{
	"%%Trailer\n"
	"%%EOF\n"
};

resource StringResource (kDocBounds, "Document Bounds", purgeable)
{
	"Document Bounds"
};

resource StringResource (kAllPaths, "All Paths", purgeable)
{
	"All Paths"
};
	
/* Alert resources */

resource StringResource (kAINeed30, purgeable)
{
	"Sorry, path export requires Photoshop 3.0 or later."
};

resource StringResource (kAITooBig, purgeable)
{
	"The document is too big for Illustrator's coordinate space."
};

resource StringResource (kAIRange, purgeable)
{
	"A point lies outside the bounds which are valid for Illustrator."
};

resource StringResource (kAIInvalidPath, purgeable)
{
	"This document contains an unsupported path format."
};

/* User interface */

resource 'DLOG' (uiID, "UI", purgeable)
	{
	{0, 0, 223, 344},
	dBoxProc,
	invisible,
	noGoAway,
	0x0,
	uiID,
	plugInName
	};

resource 'dctb' (uiID, "UI", purgeable)
	{
	{}
	};

resource 'DITL' (uiID, "UI", purgeable)
	{
		{
		{161, 252, 181, 332}, Button { enabled, "Save" },
		{130, 252, 150, 332}, Button { enabled, "Cancel" },
		{0, 0, 0, 0},		  HelpItem { disabled, HMScanhdlg { -6043 } },
		{8, 235, 24, 337},	  UserItem { enabled },
		{32, 252, 52, 332},   Button { enabled, "Eject" },
		{60, 252, 80, 332},   Button { enabled, "Desktop" },
		{29, 12, 127, 230},   UserItem { enabled },
		{6, 12, 25, 230},	  UserItem { enabled },
		{119, 250, 120, 334}, Picture { disabled, 11 },
		{157, 15, 173, 227},  EditText { enabled, "" },
		{136, 15, 152, 227},  StaticText { disabled, "Export paths to file:" },
		{88, 252, 108, 332},  UserItem { disabled },
		{194, 14, 213, 330},   Control { enabled, uiID }
		}
	};

resource 'CNTL' (uiID, "UI", purgeable)
	{
	{194, 14, 213, 330},	/* Enclosing rectangle from above. */
	popupTitleLeftJust,
	visible,
	49,
	uiID,
	popupMenuCDEFProc,
	0,
	"Write:"
	};
	
resource 'MENU' (uiID, "UI", preload)
	{
	uiID,
	textMenuProc,
	0b1111111111111111111111111110101,
	enabled,
	"Write",
		{
		"Document Bounds",				noIcon,	noKey,	noMark,	plain;
		"-",							noIcon,	noKey,	noMark,	plain;
		"All Paths",					noIcon,	noKey,	noMark,	plain;
		"-",							noIcon,	noKey,	noMark,	plain;
		}
	};
	
/* scripting dictionary resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		vendorName,											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{},													/* structure for filters */
		{													/* non-filter plug-in class here */
			vendorName " IllustratorExport",				/* unique class name */
			ourClassID,										/* class ID, must be unique or Suite ID */
			"IllustratorExport plug-in",					/* optional description */
			{												/* define inheritance */
				"<Inheritance>",							/* must be exactly this */
				keyInherits,								/* must be keyInherits */
				classExport,								/* parent: Format, Import, Export */
				"parent class export",						/* optional description */
				flagsSingleProperty,						/* if properties, list below */
				
				"in",										/* our path */
				keyIn,										/* common key */
				typePlatformFilePath,						/* correct path for platform */
				"file destination",							/* optional description */
				flagsSingleProperty,
				
				vendorName " path",							// name of path
				keyOurPath,									// our own key
				typeText,									// text
				"path",										// you get the idea
				flagsSingleProperty
				/* no more properties */
			},
			{}, /* elements (not supported) */
			/* class descriptions */
		},
		{}, /* comparison ops (not supported) */
		{}	/* any enumerations */
	}
};
