/*
	File: IllustratorExportScripting.c

	Copyright (c) 1993-6, Adobe Systems Incorporated.
	All rights reserved.

	C source file for scripting functions for Illustrator export example.
*/

#include "IllustratorExport.h"

/*****************************************************************************/
/* Checks the parameters against scripting-returned parameters, if any, and
   updates our parameters to match ones given to us by the scripting system. */

Boolean ReadScriptParams (GPtr globals)
{
	PIReadDescriptor			token = NULL;
	DescriptorKeyID				key = NULLID;
	DescriptorTypeID			type = NULLID;
	DescriptorKeyIDArray		array = { keyIn, keyOurPath, NULLID };
	int32						flags = 0;
	Handle						h = NULL; // for pathname
	OSErr						stickyError = noErr;
	Boolean						returnValue = true;
	
	if (DescriptorAvailable())
	{
		token = OpenReader(array);
		if (token)
		{
			while (PIGetKey(token, &key, &type, &flags))
			{
				switch (key)
				{
					case keyIn:
						PIGetAlias(token, &(Handle)gAliasHandle);
						break;
						// ignore all other cases and classes
					
					case keyOurPath:
						PIGetText(token, &h);
						if (h != NULL)
						{ // passed us a handle
							gPopUpValue = Handle2PopUp(globals, h);
							PIDisposeHandle(h); // dispose what we're given
							h = NULL; // just in case
						}
						break;
				}
			}

			stickyError = CloseReader(&token); // we're done, dispose.
			
			if (stickyError)
			{
				if (stickyError == errMissingParameter)
					;
					/* (descriptorKeyIDArray != NULL) Missing a parameter.
				       Walk IDarray for which one. */
				else
					gResult = stickyError; // real error occurred
			}
		}
		gQueryForParameters = returnValue = PlayDialog();
		/* return TRUE if want to show our Dialog */
	}
	return returnValue;
}

/*****************************************************************************/
/* Writes our parameters to the scripting system. */

OSErr WriteScriptParams (GPtr globals)
{
	PIWriteDescriptor			token = nil;
	Handle						h = NULL; // for path
	OSErr						gotErr = noErr;
				
	if (DescriptorAvailable())
	{
		token = OpenWriter();
		if (token)
		{					
			PIPutAlias(token, keyIn, (Handle)gAliasHandle);
			
			h = PopUp2Handle(globals, gPopUpValue);
			if (h != NULL) PIPutText(token, keyOurPath, h);
			gotErr = CloseWriter(&token); // closes and sets to dialogOptional
			
			if (gAliasHandle != NULL)
			{ // dispose the handle we created
				PIDisposeHandle((Handle)gAliasHandle);
				gAliasHandle = NULL;
			}
			
			if (h != NULL)
			{ // we created it or were handed it.  It's our job to dispose.
				PIDisposeHandle(h);
				h = NULL;
			}
			/* done.  Now pass descriptor on to Photoshop */
		}
	}
	return gotErr;
}

/*****************************************************************************/
// Maps gPopUp to a string then returns the string as a handle

Handle PopUp2Handle (GPtr globals, short popup)
{
	Str255 s = "";
	
	switch (popup)
	{
		case iDocBounds:
			PIGetString(s, kDocBounds);
			break;
		case iAllPaths:
			PIGetString(s, kAllPaths);
			break;
		default:
			if (popup >= iPathStart)
				GetPathName(globals, popup - iPathStart, s);
			break;
	}
	if (s[0] > 0)
		return PIString2Handle(s);
	else return NULL;
}

/*****************************************************************************/
// Maps string to popup values and returns the index

short Handle2PopUp (GPtr globals, Handle h)
{
	Str255 s = "";
	Str255 s2 = "";
	int32 pathCount = 0;
	int32 pathIndex = 0;
	short retn = 0;
	
	PIHandle2String(h, s2);
	
	PIGetString(s, kDocBounds);
	if (PISMatch(s, (char *)s2) == gotMatch)
		retn=iDocBounds; // document bounds
	else
	{ // keep checking
		PIGetString(s, kAllPaths);
		if (PISMatch(s, (char *)s2) == gotMatch)
			retn=iAllPaths; // all paths
		else
		{ // keep checking
			GetPathCount(globals, &pathCount);
			if (pathCount < 1)
				retn = 0; // no paths
			else
			{ // keep checking
				do
				{	
					if (GetPathName (globals, pathIndex, s))
					{ // got a name
						if (PISMatch(s, (char *)s2) == gotMatch)
						{
							retn = (short)(iPathStart+pathIndex); // extra menu items
							pathIndex = pathCount; // skip to end
						}
					}
				} while (++pathIndex < pathCount);
			}
		}
	}
	return retn;
}
